@@ -24,6 +24,7 @@
 #include "postgres.h"
 
 #include "miscadmin.h"
+#include "port.h"
 #include "safe_lib.h"
 
 #include "access/heapam.h"
@@ -42,19 +43,6 @@
 #include "executor/spi.h"
 #include "lib/stringinfo.h"
 #include "nodes/execnodes.h"
-
-#include "citus_version.h"
-
-#include "columnar/columnar.h"
-#include "columnar/columnar_storage.h"
-#include "columnar/columnar_version_compat.h"
-
-#include "distributed/listutils.h"
-#if PG_VERSION_NUM >= PG_VERSION_16
-#include "parser/parse_relation.h"
-#endif
-#include "port.h"
-
 #include "storage/fd.h"
 #include "storage/lmgr.h"
 #include "storage/procarray.h"
@@ -64,7 +52,18 @@
 #include "utils/lsyscache.h"
 #include "utils/memutils.h"
 #include "utils/rel.h"
+
+#include "citus_version.h"
+#include "pg_version_constants.h"
+
+#include "columnar/columnar.h"
+#include "columnar/columnar_storage.h"
+#include "columnar/columnar_version_compat.h"
+
+#include "distributed/listutils.h"
+
 #if PG_VERSION_NUM >= PG_VERSION_16
+#include "parser/parse_relation.h"
 #include "storage/relfilelocator.h"
 #include "utils/relfilenumbermap.h"
 #else
@@ -657,9 +656,7 @@ SaveStripeSkipList(RelFileLocator relfilelocator, uint64 stripe,
 				nulls[Anum_columnar_chunk_maximum_value - 1] = true;
 			}
 
-			PushActiveSnapshot(GetTransactionSnapshot());
 			InsertTupleAndEnforceConstraints(modifyState, values, nulls);
-			PopActiveSnapshot();
 		}
 	}
 
