@@ -209,12 +209,9 @@ static void ReplaceTable(Oid sourceId, Oid targetId, List *justBeforeDropCommand
 static bool HasAnyGeneratedStoredColumns(Oid relationId);
 static List * GetNonGeneratedStoredColumnNameList(Oid relationId);
 static void CheckAlterDistributedTableConversionParameters(TableConversionState *con);
-static char * CreateWorkerChangeSequenceDependencyCommand(char *sequenceSchemaName,
-														  char *sequenceName,
-														  char *sourceSchemaName,
-														  char *sourceName,
-														  char *targetSchemaName,
-														  char *targetName);
+static char * CreateWorkerChangeSequenceDependencyCommand(char *qualifiedSequeceName,
+														  char *qualifiedSourceName,
+														  char *qualifiedTargetName);
 static void ErrorIfMatViewSizeExceedsTheLimit(Oid matViewOid);
 static char * CreateMaterializedViewDDLCommand(Oid matViewOid);
 static char * GetAccessMethodForMatViewIfExists(Oid viewOid);
@@ -417,7 +414,7 @@ UndistributeTables(List *relationIdList)
 	 */
 	List *originalForeignKeyRecreationCommands = NIL;
 	Oid relationId = InvalidOid;
-	foreach_oid(relationId, relationIdList)
+	foreach_declared_oid(relationId, relationIdList)
 	{
 		List *fkeyCommandsForRelation =
 			GetFKeyCreationCommandsRelationInvolvedWithTableType(relationId,
@@ -791,19 +788,21 @@ ConvertTableInternal(TableConversionState *con)
 		justBeforeDropCommands = lappend(justBeforeDropCommands, detachFromParentCommand);
 	}
 
+	char *qualifiedRelationName = quote_qualified_identifier(con->schemaName,
+															 con->relationName);
+
 	if (PartitionedTable(con->relationId))
 	{
 		if (!con->suppressNoticeMessages)
 		{
 			ereport(NOTICE, (errmsg("converting the partitions of %s",
-									quote_qualified_identifier(con->schemaName,
-															   con->relationName))));
+									qualifiedRelationName)));
 		}
 
 		List *partitionList = PartitionList(con->relationId);
 
 		Oid partitionRelationId = InvalidOid;
-		foreach_oid(partitionRelationId, partitionList)
+		foreach_declared_oid(partitionRelationId, partitionList)
 		{
 			char *tableQualifiedName = generate_qualified_relation_name(
 				partitionRelationId);
@@ -870,13 +869,11 @@ ConvertTableInternal(TableConversionState *con)
 
 	if (!con->suppressNoticeMessages)
 	{
-		ereport(NOTICE, (errmsg("creating a new table for %s",
-								quote_qualified_identifier(con->schemaName,
-														   con->relationName))));
+		ereport(NOTICE, (errmsg("creating a new table for %s", qualifiedRelationName)));
 	}
 
 	TableDDLCommand *tableCreationCommand = NULL;
-	foreach_ptr(tableCreationCommand, preLoadCommands)
+	foreach_declared_ptr(tableCreationCommand, preLoadCommands)
 	{
 		Assert(CitusIsA(tableCreationCommand, TableDDLCommand));
 
@@ -950,7 +947,7 @@ ConvertTableInternal(TableConversionState *con)
 				 con->suppressNoticeMessages);
 
 	TableDDLCommand *tableConstructionCommand = NULL;
-	foreach_ptr(tableConstructionCommand, postLoadCommands)
+	foreach_declared_ptr(tableConstructionCommand, postLoadCommands)
 	{
 		Assert(CitusIsA(tableConstructionCommand, TableDDLCommand));
 		char *tableConstructionSQL = GetTableDDLCommand(tableConstructionCommand);
@@ -968,7 +965,7 @@ ConvertTableInternal(TableConversionState *con)
 	MemoryContext oldContext = MemoryContextSwitchTo(citusPerPartitionContext);
 
 	char *attachPartitionCommand = NULL;
-	foreach_ptr(attachPartitionCommand, attachPartitionCommands)
+	foreach_declared_ptr(attachPartitionCommand, attachPartitionCommands)
 	{
 		MemoryContextReset(citusPerPartitionContext);
 
@@ -993,14 +990,12 @@ ConvertTableInternal(TableConversionState *con)
 
 		/* For now we only support cascade to colocation for alter_distributed_table UDF */
 		Assert(con->conversionType == ALTER_DISTRIBUTED_TABLE);
-		foreach_oid(colocatedTableId, con->colocatedTableList)
+		foreach_declared_oid(colocatedTableId, con->colocatedTableList)
 		{
 			if (colocatedTableId == con->relationId)
 			{
 				continue;
 			}
-			char *qualifiedRelationName = quote_qualified_identifier(con->schemaName,
-																	 con->relationName);
 
 			TableConversionParameters cascadeParam = {
 				.relationId = colocatedTableId,
@@ -1023,7 +1018,7 @@ ConvertTableInternal(TableConversionState *con)
 		if (con->cascadeToColocated != CASCADE_TO_COLOCATED_NO_ALREADY_CASCADED)
 		{
 			char *foreignKeyCommand = NULL;
-			foreach_ptr(foreignKeyCommand, foreignKeyCommands)
+			foreach_declared_ptr(foreignKeyCommand, foreignKeyCommands)
 			{
 				ExecuteQueryViaSPI(foreignKeyCommand, SPI_OK_UTILITY);
 			}
@@ -1059,7 +1054,7 @@ CopyTableConversionReturnIntoCurrentContext(TableConversionReturn *tableConversi
 		tableConversionReturnCopy = palloc0(sizeof(TableConversionReturn));
 		List *copyForeignKeyCommands = NIL;
 		char *foreignKeyCommand = NULL;
-		foreach_ptr(foreignKeyCommand, tableConversionReturn->foreignKeyCommands)
+		foreach_declared_ptr(foreignKeyCommand, tableConversionReturn->foreignKeyCommands)
 		{
 			char *copyForeignKeyCommand = MemoryContextStrdup(CurrentMemoryContext,
 															  foreignKeyCommand);
@@ -1134,7 +1129,7 @@ DropIndexesNotSupportedByColumnar(Oid relationId, bool suppressNoticeMessages)
 	RelationClose(columnarRelation);
 
 	Oid indexId = InvalidOid;
-	foreach_oid(indexId, indexIdList)
+	foreach_declared_oid(indexId, indexIdList)
 	{
 		char *indexAmName = GetIndexAccessMethodName(indexId);
 		if (extern_ColumnarSupportsIndexAM(indexAmName))
@@ -1394,7 +1389,7 @@ CreateTableConversion(TableConversionParameters *params)
 		 * since they will be handled separately.
 		 */
 		Oid colocatedTableId = InvalidOid;
-		foreach_oid(colocatedTableId, colocatedTableList)
+		foreach_declared_oid(colocatedTableId, colocatedTableList)
 		{
 			if (PartitionTable(colocatedTableId))
 			{
@@ -1610,7 +1605,7 @@ DoesCascadeDropUnsupportedObject(Oid classId, Oid objectId, HTAB *nodeMap)
 																	 targetObjectId);
 
 	HeapTuple depTup = NULL;
-	foreach_ptr(depTup, dependencyTupleList)
+	foreach_declared_ptr(depTup, dependencyTupleList)
 	{
 		Form_pg_depend pg_depend = (Form_pg_depend) GETSTRUCT(depTup);
 
@@ -1650,7 +1645,7 @@ GetViewCreationCommandsOfTable(Oid relationId)
 	List *commands = NIL;
 
 	Oid viewOid = InvalidOid;
-	foreach_oid(viewOid, views)
+	foreach_declared_oid(viewOid, views)
 	{
 		StringInfo query = makeStringInfo();
 
@@ -1688,7 +1683,7 @@ WrapTableDDLCommands(List *commandStrings)
 	List *tableDDLCommands = NIL;
 
 	char *command = NULL;
-	foreach_ptr(command, commandStrings)
+	foreach_declared_ptr(command, commandStrings)
 	{
 		tableDDLCommands = lappend(tableDDLCommands, makeTableDDLCommandString(command));
 	}
@@ -1750,9 +1745,7 @@ CreateMaterializedViewDDLCommand(Oid matViewOid)
 {
 	StringInfo query = makeStringInfo();
 
-	char *viewName = get_rel_name(matViewOid);
-	char *schemaName = get_namespace_name(get_rel_namespace(matViewOid));
-	char *qualifiedViewName = quote_qualified_identifier(schemaName, viewName);
+	char *qualifiedViewName = generate_qualified_relation_name(matViewOid);
 
 	/* here we need to get the access method of the view to recreate it */
 	char *accessMethodName = GetAccessMethodForMatViewIfExists(matViewOid);
@@ -1801,9 +1794,8 @@ ReplaceTable(Oid sourceId, Oid targetId, List *justBeforeDropCommands,
 			 bool suppressNoticeMessages)
 {
 	char *sourceName = get_rel_name(sourceId);
-	char *targetName = get_rel_name(targetId);
-	Oid schemaId = get_rel_namespace(sourceId);
-	char *schemaName = get_namespace_name(schemaId);
+	char *qualifiedSourceName = generate_qualified_relation_name(sourceId);
+	char *qualifiedTargetName = generate_qualified_relation_name(targetId);
 
 	StringInfo query = makeStringInfo();
 
@@ -1811,8 +1803,7 @@ ReplaceTable(Oid sourceId, Oid targetId, List *justBeforeDropCommands,
 	{
 		if (!suppressNoticeMessages)
 		{
-			ereport(NOTICE, (errmsg("moving the data of %s",
-									quote_qualified_identifier(schemaName, sourceName))));
+			ereport(NOTICE, (errmsg("moving the data of %s", qualifiedSourceName)));
 		}
 
 		if (!HasAnyGeneratedStoredColumns(sourceId))
@@ -1822,8 +1813,7 @@ ReplaceTable(Oid sourceId, Oid targetId, List *justBeforeDropCommands,
 			 * "INSERT INTO .. SELECT *"".
 			 */
 			appendStringInfo(query, "INSERT INTO %s SELECT * FROM %s",
-							 quote_qualified_identifier(schemaName, targetName),
-							 quote_qualified_identifier(schemaName, sourceName));
+							 qualifiedTargetName, qualifiedSourceName);
 		}
 		else
 		{
@@ -1838,9 +1828,8 @@ ReplaceTable(Oid sourceId, Oid targetId, List *justBeforeDropCommands,
 			char *insertColumnString = StringJoin(nonStoredColumnNameList, ',');
 			appendStringInfo(query,
 							 "INSERT INTO %s (%s) OVERRIDING SYSTEM VALUE SELECT %s FROM %s",
-							 quote_qualified_identifier(schemaName, targetName),
-							 insertColumnString, insertColumnString,
-							 quote_qualified_identifier(schemaName, sourceName));
+							 qualifiedTargetName, insertColumnString,
+							 insertColumnString, qualifiedSourceName);
 		}
 
 		ExecuteQueryViaSPI(query->data, SPI_OK_INSERT);
@@ -1851,7 +1840,7 @@ ReplaceTable(Oid sourceId, Oid targetId, List *justBeforeDropCommands,
 	 */
 	List *ownedSequences = getOwnedSequences_internal(sourceId, 0, DEPENDENCY_AUTO);
 	Oid sequenceOid = InvalidOid;
-	foreach_oid(sequenceOid, ownedSequences)
+	foreach_declared_oid(sequenceOid, ownedSequences)
 	{
 		changeDependencyFor(RelationRelationId, sequenceOid,
 							RelationRelationId, sourceId, targetId);
@@ -1864,14 +1853,11 @@ ReplaceTable(Oid sourceId, Oid targetId, List *justBeforeDropCommands,
 		 */
 		if (ShouldSyncTableMetadata(targetId))
 		{
-			Oid sequenceSchemaOid = get_rel_namespace(sequenceOid);
-			char *sequenceSchemaName = get_namespace_name(sequenceSchemaOid);
-			char *sequenceName = get_rel_name(sequenceOid);
+			char *qualifiedSequenceName = generate_qualified_relation_name(sequenceOid);
 			char *workerChangeSequenceDependencyCommand =
-				CreateWorkerChangeSequenceDependencyCommand(sequenceSchemaName,
-															sequenceName,
-															schemaName, sourceName,
-															schemaName, targetName);
+				CreateWorkerChangeSequenceDependencyCommand(qualifiedSequenceName,
+															qualifiedSourceName,
+															qualifiedTargetName);
 			SendCommandToWorkersWithMetadata(workerChangeSequenceDependencyCommand);
 		}
 		else if (ShouldSyncTableMetadata(sourceId))
@@ -1887,32 +1873,30 @@ ReplaceTable(Oid sourceId, Oid targetId, List *justBeforeDropCommands,
 	}
 
 	char *justBeforeDropCommand = NULL;
-	foreach_ptr(justBeforeDropCommand, justBeforeDropCommands)
+	foreach_declared_ptr(justBeforeDropCommand, justBeforeDropCommands)
 	{
 		ExecuteQueryViaSPI(justBeforeDropCommand, SPI_OK_UTILITY);
 	}
 
 	if (!suppressNoticeMessages)
 	{
-		ereport(NOTICE, (errmsg("dropping the old %s",
-								quote_qualified_identifier(schemaName, sourceName))));
+		ereport(NOTICE, (errmsg("dropping the old %s", qualifiedSourceName)));
 	}
 
 	resetStringInfo(query);
 	appendStringInfo(query, "DROP %sTABLE %s CASCADE",
 					 IsForeignTable(sourceId) ? "FOREIGN " : "",
-					 quote_qualified_identifier(schemaName, sourceName));
+					 qualifiedSourceName);
 	ExecuteQueryViaSPI(query->data, SPI_OK_UTILITY);
 
 	if (!suppressNoticeMessages)
 	{
-		ereport(NOTICE, (errmsg("renaming the new table to %s",
-								quote_qualified_identifier(schemaName, sourceName))));
+		ereport(NOTICE, (errmsg("renaming the new table to %s", qualifiedSourceName)));
 	}
 
 	resetStringInfo(query);
 	appendStringInfo(query, "ALTER TABLE %s RENAME TO %s",
-					 quote_qualified_identifier(schemaName, targetName),
+					 qualifiedTargetName,
 					 quote_identifier(sourceName));
 	ExecuteQueryViaSPI(query->data, SPI_OK_UTILITY);
 }
@@ -2003,7 +1987,7 @@ CheckAlterDistributedTableConversionParameters(TableConversionState *con)
 		Oid colocatedTableOid = InvalidOid;
 		text *colocateWithText = cstring_to_text(con->colocateWith);
 		Oid colocateWithTableOid = ResolveRelationId(colocateWithText, false);
-		foreach_oid(colocatedTableOid, con->colocatedTableList)
+		foreach_declared_oid(colocatedTableOid, con->colocatedTableList)
 		{
 			if (colocateWithTableOid == colocatedTableOid)
 			{
@@ -2172,18 +2156,13 @@ CheckAlterDistributedTableConversionParameters(TableConversionState *con)
  * worker_change_sequence_dependency query with the parameters.
  */
 static char *
-CreateWorkerChangeSequenceDependencyCommand(char *sequenceSchemaName, char *sequenceName,
-											char *sourceSchemaName, char *sourceName,
-											char *targetSchemaName, char *targetName)
+CreateWorkerChangeSequenceDependencyCommand(char *qualifiedSequeceName,
+											char *qualifiedSourceName,
+											char *qualifiedTargetName)
 {
-	char *qualifiedSchemaName = quote_qualified_identifier(sequenceSchemaName,
-														   sequenceName);
-	char *qualifiedSourceName = quote_qualified_identifier(sourceSchemaName, sourceName);
-	char *qualifiedTargetName = quote_qualified_identifier(targetSchemaName, targetName);
-
 	StringInfo query = makeStringInfo();
 	appendStringInfo(query, "SELECT worker_change_sequence_dependency(%s, %s, %s)",
-					 quote_literal_cstr(qualifiedSchemaName),
+					 quote_literal_cstr(qualifiedSequeceName),
 					 quote_literal_cstr(qualifiedSourceName),
 					 quote_literal_cstr(qualifiedTargetName));
 
@@ -2235,7 +2214,7 @@ WillRecreateForeignKeyToReferenceTable(Oid relationId,
 	{
 		List *colocatedTableList = ColocatedTableList(relationId);
 		Oid colocatedTableOid = InvalidOid;
-		foreach_oid(colocatedTableOid, colocatedTableList)
+		foreach_declared_oid(colocatedTableOid, colocatedTableList)
 		{
 			if (HasForeignKeyToReferenceTable(colocatedTableOid))
 			{
@@ -2263,7 +2242,7 @@ WarningsForDroppingForeignKeysWithDistributedTables(Oid relationId)
 	List *foreignKeys = list_concat(referencingForeingKeys, referencedForeignKeys);
 
 	Oid foreignKeyOid = InvalidOid;
-	foreach_oid(foreignKeyOid, foreignKeys)
+	foreach_declared_oid(foreignKeyOid, foreignKeys)
 	{
 		ereport(WARNING, (errmsg("foreign key %s will be dropped",
 								 get_constraint_name(foreignKeyOid))));
