@@ -23,6 +23,18 @@
 #include "nodes/makefuncs.h"
 #include "nodes/nodeFuncs.h"
 #include "nodes/pg_list.h"
+#include "optimizer/optimizer.h"
+#include "optimizer/pathnode.h"
+#include "optimizer/plancat.h"
+#include "optimizer/planmain.h"
+#include "optimizer/planner.h"
+#include "parser/parse_type.h"
+#include "parser/parsetree.h"
+#include "utils/builtins.h"
+#include "utils/datum.h"
+#include "utils/lsyscache.h"
+#include "utils/memutils.h"
+#include "utils/syscache.h"
 
 #include "pg_version_constants.h"
 
@@ -54,24 +66,13 @@
 #include "distributed/recursive_planning.h"
 #include "distributed/shard_utils.h"
 #include "distributed/shardinterval_utils.h"
-#include "distributed/utils/citus_stat_tenants.h"
+#include "distributed/stats/stat_tenants.h"
 #include "distributed/version_compat.h"
 #include "distributed/worker_shard_visibility.h"
+
 #if PG_VERSION_NUM >= PG_VERSION_16
 #include "parser/parse_relation.h"
 #endif
-#include "optimizer/optimizer.h"
-#include "optimizer/pathnode.h"
-#include "optimizer/plancat.h"
-#include "optimizer/planmain.h"
-#include "optimizer/planner.h"
-#include "parser/parse_type.h"
-#include "parser/parsetree.h"
-#include "utils/builtins.h"
-#include "utils/datum.h"
-#include "utils/lsyscache.h"
-#include "utils/memutils.h"
-#include "utils/syscache.h"
 
 
 /* RouterPlanType is used to determine the router plan to invoke */
@@ -151,9 +152,10 @@ static RouterPlanType GetRouterPlanType(Query *query,
 										bool hasUnresolvedParams);
 static void ConcatenateRTablesAndPerminfos(PlannedStmt *mainPlan,
 										   PlannedStmt *concatPlan);
-static bool CheckPostPlanDistribution(DistributedPlanningContext *planContext,
-									  bool isDistributedQuery,
-									  List *rangeTableList);
+static bool CheckPostPlanDistribution(bool isDistributedQuery,
+									  Query *origQuery,
+									  List *rangeTableList,
+									  Query *plannedQuery);
 
 /* Distributed planner hook */
 PlannedStmt *
@@ -274,9 +276,10 @@ distributed_planner(Query *parse,
 			planContext.plan = standard_planner(planContext.query, NULL,
 												planContext.cursorOptions,
 												planContext.boundParams);
-			needsDistributedPlanning = CheckPostPlanDistribution(&planContext,
-																 needsDistributedPlanning,
-																 rangeTableList);
+			needsDistributedPlanning = CheckPostPlanDistribution(needsDistributedPlanning,
+																 planContext.originalQuery,
+																 rangeTableList,
+																 planContext.query);
 
 			if (needsDistributedPlanning)
 			{
@@ -2737,13 +2740,12 @@ WarnIfListHasForeignDistributedTable(List *rangeTableList)
 
 
 static bool
-CheckPostPlanDistribution(DistributedPlanningContext *planContext, bool
-						  isDistributedQuery, List *rangeTableList)
+CheckPostPlanDistribution(bool isDistributedQuery,
+						  Query *origQuery, List *rangeTableList,
+						  Query *plannedQuery)
 {
 	if (isDistributedQuery)
 	{
-		Query *origQuery = planContext->originalQuery;
-		Query *plannedQuery = planContext->query;
 		Node *origQuals = origQuery->jointree->quals;
 		Node *plannedQuals = plannedQuery->jointree->quals;
 
@@ -2762,23 +2764,6 @@ CheckPostPlanDistribution(DistributedPlanningContext *planContext, bool
 		 */
 		if (origQuals != NULL && plannedQuals == NULL)
 		{
-			bool planHasDistTable = ListContainsDistributedTableRTE(
-				planContext->plan->rtable, NULL);
-
-			/*
-			 * If the Postgres plan has a distributed table, we know for sure that
-			 * the query requires distributed planning.
-			 */
-			if (planHasDistTable)
-			{
-				return true;
-			}
-
-			/*
-			 * Otherwise, if the query has less range table entries after Postgres,
-			 * planning, we should re-evaluate the distribution of the query. Postgres
-			 * may have optimized away all citus tables, per issues 7782, 7783.
-			 */
 			List *rtesPostPlan = ExtractRangeTableEntryList(plannedQuery);
 			if (list_length(rtesPostPlan) < list_length(rangeTableList))
 			{
