@@ -21,7 +21,8 @@
 /* Function declarations for transmitting files between two nodes */
 extern void RedirectCopyDataToRegularFile(const char *filename);
 extern void SendRegularFile(const char *filename);
-extern File FileOpenForTransmit(const char *filename, int fileFlags, int fileMode);
+extern File FileOpenForTransmit(const char *filename, int fileFlags);
+extern File FileOpenForTransmitPerm(const char *filename, int fileFlags, int fileMode);
 
 
 #endif   /* TRANSMIT_H */
