@@ -90,7 +90,6 @@ my $workerCount = 2;
 my $serversAreShutdown = "TRUE";
 my $usingWindows = 0;
 my $mitmPid = 0;
-my $workerCount = 2;
 
 if ($Config{osname} eq "MSWin32")
 {
@@ -493,6 +492,8 @@ push(@pgOptions, "citus.stat_statements_track = 'all'");
 push(@pgOptions, "citus.enable_change_data_capture=on");
 push(@pgOptions, "citus.stat_tenants_limit = 2");
 push(@pgOptions, "citus.stat_tenants_track = 'ALL'");
+push(@pgOptions, "citus.enable_stat_counters=on");
+push(@pgOptions, "citus.superuser = 'postgres'");
 
 # Some tests look at shards in pg_class, make sure we can usually see them:
 push(@pgOptions, "citus.show_shards_for_app_name_prefixes='pg_regress'");
@@ -637,7 +638,7 @@ for my $port (@followerWorkerPorts)
 	}
 }
 
-for my $tablespace ("ts0", "ts1", "ts2")
+for my $tablespace ("ts0", "ts1", "ts2", "ts3", "ts4", "ts5")
 {
 	if (-e catfile($TMP_CHECKDIR, $tablespace))
 	{
@@ -1199,4 +1200,3 @@ else {
 	die "Failed in ". ($endTime - $startTime)." seconds. \n";
 
 }
-
