@@ -297,8 +297,15 @@ SET search_path TO public;
 SHOW server_version \gset
 SELECT substring(:'server_version', '\d+')::int >= 16 AS server_version_ge_16
 \gset
+SELECT substring(:'server_version', '\d+')::int >= 17 AS server_version_ge_17
+\gset
 
-\if :server_version_ge_16
+\if :server_version_ge_17
+-- PG17 renamed colliculocale to colllocale
+-- Relevant PG commit:
+-- https://github.com/postgres/postgres/commit/f696c0cd5f299f1b51e214efc55a22a782cc175d
+SELECT quote_ident((SELECT CASE WHEN datlocprovider='i' THEN datlocale ELSE datcollate END FROM pg_database WHERE datname = current_database())) as current_locale \gset
+\elif :server_version_ge_16
 -- In PG16, read-only server settings lc_collate and lc_ctype are removed
 -- Relevant PG commit: b0f6c437160db640d4ea3e49398ebc3ba39d1982
 SELECT quote_ident((SELECT CASE WHEN datlocprovider='i' THEN daticulocale ELSE datcollate END FROM pg_database WHERE datname = current_database())) as current_locale \gset
