@@ -608,5 +608,5 @@ ALTER SEQUENCE pg_catalog.pg_dist_colocationid_seq RESTART 100;
 
 ALTER SEQUENCE pg_catalog.pg_dist_placement_placementid_seq RESTART :last_placement_id;
 
-SELECT 	citus_set_coordinator_host('localhost');
-
+-- make sure we don't have any replication objects leftover on the nodes
+SELECT public.wait_for_resource_cleanup();
